#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：dup1
@File    ：encode.py
@Author  ：Zhl
@Date    ：2021/9/8 6:24 下午
"""
import json


def feat2idx(cols_set):
    """
    :param cols_set:
    :return:
    """
    cols_idx = {col: {} for col in cols_set.keys()}
    cols_maxlen = {col: 0 for col in cols_set.keys()}
    for col, vals in cols_set.items():
        vals = sorted(vals)
        skip = len(str(len(vals))) * 10
        for i, val in enumerate(vals):
            cols_idx[col][val] = i
        cols_maxlen[col] = i + skip
    return cols_idx, cols_maxlen


def encode2idx(idx_file, sep, cols, maps):
    """
    :param idx_file:
    :param cols:
    :param maps:
    :return:
    """
    fr = open(idx_file, "r")
    lines = fr.readlines()
    cols_set = {col: set() for col in cols}
    maps_val = {m["map"]: {} for m in maps}
    for line in lines:
        cells = line.replace("\n", "").split(sep)
        row = dict(zip(cols, ["-1" if c.lower().strip() in set(["null", ""]) else c for c in cells]))
        for col in cols:
            cols_set[col].add(row[col])
        for i in range(len(maps)):
            val = "_".join([row[j] for j in maps[i]["values"]])
            if row[maps[i]["key"]] not in maps_val[maps[i]["map"]]:
                v = set()
                v.add(val)
                maps_val[maps[i]["map"]][row[maps[i]["key"]]] = v
            else:
                maps_val[maps[i]["map"]][row[maps[i]["key"]]].add(val)
    cols_idx, cols_maxlen = feat2idx(cols_set)
    return cols_maxlen, cols_idx, maps_val


def merge2dump(cols_maxlen, cols_idx, maps_val, dump_file):
    """
    @param ret2idx:
    @param ret2feat:
    @return:
    """
    encode_dict = {"cols_maxlen": cols_maxlen}
    encode_dict.update(cols_idx)
    for m, vals in maps_val.items():
        for k, v in vals.items():
            maps_val[m][k] = sorted(list(v))
    encode_dict.update(maps_val)
    json.dump(encode_dict, open(dump_file, "w"), indent=4, separators=(',', ':'))


cols = ["act", "event_id", "cid3", "pwid"]
maps = [{"map": "cid3_pwid_tab", "key": "cid3", "values": ["pwid"]}]
idx_file = "../data/encode_lbs_prefer.txt"
sep = "\t"
dump_file = "../data/encode_idx_conf2.json"
cols_maxlen, cols_idx, maps_val = encode2idx(idx_file, sep, cols, maps)
merge2dump(cols_maxlen, cols_idx, maps_val, dump_file)
