import os
import shutil

import tensorflow as tf


class Export:
    def __init__(self):
        pass

    def export_pb_from_ckpt(self, ckpt_in_path, pb_out_path, input_node_names, output_node_names):
        """
        :param ckpt_in_path:
        :param pb_out_path:
        :param input_node_names:
        :param output_node_names:
        :return:
        """
        pbfile_out_path = f"{pb_out_path}/model.pb"
        export_dir = os.path.dirname(pbfile_out_path)
        if os.path.exists(export_dir): shutil.rmtree(export_dir)
        os.makedirs(export_dir)
        graph = tf.Graph()
        with graph.as_default():
            session_conf = tf.ConfigProto(allow_soft_placement=True, log_device_placement=False)
            sess = tf.Session(config=session_conf)
            with sess.as_default():
                # import something like meta graph def, graph def, save def.
                moudke_file = tf.train.latest_checkpoint(ckpt_in_path)
                saver = tf.train.import_meta_graph(moudke_file + ".meta", clear_devices=True)
                # recover variables information.
                saver.restore(sess, moudke_file)
                print("***", graph.get_name_scope())
                # convert variable into constant ones and remove ops related to
                # loading and saving the variables. support to omit some variables.
                converted_graph_def = tf.graph_util.convert_variables_to_constants(sess, graph.as_graph_def(),
                                                                                   output_node_names)
                # extract the subgraph that can reach any of the nodes in dest_nodes.
                g = tf.graph_util.extract_sub_graph(converted_graph_def, output_node_names)
                # remove nodes like identity and checknumerics that are only used during training.
                g = tf.graph_util.remove_training_nodes(g, protected_nodes=input_node_names + output_node_names)
                with tf.gfile.FastGFile(pbfile_out_path, mode='wb') as fw:
                    fw.write(g.SerializeToString())

    def export_savedmodel_from_pb(self, pb_in_path, savedmodel_out_path, input_signature_map, output_signature_map):
        """
        :param pb_in_path:
        :param savedmodel_out_path:
        :param input_signature_map:
        :param output_signature_map:
        :return:
        """
        pbfile_out_path = f"{pb_in_path}/model.pb"
        input_signature_node_map = dict()
        output_signature_node_map = dict()
        input_signature_names = list(input_signature_map.keys())
        output_signature_names = list(output_signature_map.keys())
        if os.path.exists(savedmodel_out_path):
            shutil.rmtree(savedmodel_out_path)
        with tf.Graph().as_default() as graph:
            main_graph_def = tf.GraphDef()
            with open(pbfile_out_path, "rb") as fr:
                main_graph_def.ParseFromString(fr.read())
            _ = tf.import_graph_def(main_graph_def)
            for op in graph.get_operations():
                print(op.name)
            # build a savedmodel that can be saved to storage, is language neutral.
            # savedmodel is a protocol buffer and saves variables and assets.
            builder = tf.saved_model.builder.SavedModelBuilder(savedmodel_out_path)
            # build tensorinfo proto from a tensor.
            for input_signature_name in input_signature_names:
                input_tensor = graph.get_tensor_by_name("import/" + input_signature_map[input_signature_name] + ":0")
                print(input_tensor.name, input_tensor.shape)
                input_tensor_info = tf.saved_model.utils.build_tensor_info(input_tensor)
                input_signature_node_map[input_signature_name] = input_tensor_info

            for output_signature_name in output_signature_names:
                output_tensor = graph.get_tensor_by_name("import/" + output_signature_map[output_signature_name] + ":0")
                print(output_tensor.name, output_tensor.shape)
                output_tensor_info = tf.saved_model.utils.build_tensor_info(output_tensor)
                output_signature_node_map[output_signature_name] = output_tensor_info
            # build a signaturedef protocol buffer.
            labeling_signature = (
                tf.saved_model.signature_def_utils.build_signature_def(
                    inputs=input_signature_node_map,
                    outputs=output_signature_node_map,
                    method_name="tensorflow/serving/predict"))
            session_conf = tf.ConfigProto(allow_soft_placement=True, log_device_placement=False)
            sess = tf.Session(config=session_conf)
            with sess.as_default():
                # fill meta_graph_def and tensor_info_def into builder.
                builder.add_meta_graph_and_variables(sess,
                                                     # tag for meta graph def.
                                                     tags=[tf.saved_model.tag_constants.SERVING],
                                                     # signature for tensor information.
                                                     signature_def_map={
                                                         tf.saved_model.signature_constants.DEFAULT_SERVING_SIGNATURE_DEF_KEY:
                                                             labeling_signature
                                                     })
                builder.save()
                print("Build Done")
