import os
import time

import numpy as np
import tensorflow as tf

from dup1.utils.log_util import pylog


def get_sess_config():
    """
    获取session配置信息
    :return:
    """
    return tf.ConfigProto(
        log_device_placement=True,
        inter_op_parallelism_threads=20,
        intra_op_parallelism_threads=20)


def trans_feat_feed(features, input_ph_dict):
    """
    :param features:
    :param input_ph_dict:
    :return:
    """
    feed_dict = {}
    for feat, ph in input_ph_dict.items():
        feed_dict[ph] = features[feat]
    return feed_dict


def save_ckpt(sess, iter, ckpt_path):
    """
    :param sess:
    :param iter:
    :param ckpt_path:
    :return:
    """
    saver = tf.train.Saver()
    os.makedirs(ckpt_path, exist_ok=True)
    ckpt_path = f"{ckpt_path}/model.ckpt-{iter}"
    saver.save(sess, ckpt_path)


class LocalTask:
    def __init__(self, reader, model, ckpt_path, batch_size=256, steps_log=32, steps_save=1024):
        """
        :param reader:
        :param model:
        :param ckpt_path:
        :param batch_size:
        :param steps_log:
        :param steps_save:
        """
        self.reader = reader
        self.model = model
        self.ckpt_path = ckpt_path
        self.batch_size = batch_size
        self.steps_log = steps_log
        self.steps_save = steps_save

    def train(self):
        """
        :return:
        """
        with tf.Session(config=get_sess_config()) as sess:
            tf.set_random_seed(9)
            end_point = self.model.end_point
            logits = end_point["logits"]
            batch_loss = self.model.build_loss(logits, self.model.input_ph_dict["label_ph"], end_point=end_point)
            trainer, global_step, _ = self.model.build_train_op()
            pylog.info("Training New Model ...")
            init = [tf.global_variables_initializer()]
            sess.run(init)
            # sess.graph.finalize()
            # some varibles to fomat print
            tic = time.time()
            log_cnt, total_loss = 1, 0.0
            pylog.info(f"train files:{self.reader.filenames}")
            for features, labels in self.reader:
                feed = trans_feat_feed(features, self.model.input_ph_dict)
                _, loss, step = sess.run([trainer, batch_loss, global_step], feed_dict=feed)
                total_loss += loss
                if step % self.steps_log == 0:
                    toc = time.time()
                    log_info = []
                    batch_cost = round((toc - tic) * 1000 / self.steps_log, 2)
                    row_cost = round((toc - tic) * 1000000 / (self.steps_log * self.batch_size), 2)
                    log_info.extend([f"row:{log_cnt * self.batch_size // 10000}w"])
                    log_info.extend([f"steps:{step // 1000}k", f"log_cnt:{log_cnt}"])
                    log_info.extend([f"loss:{np.round(total_loss / log_cnt, 5)}"])
                    log_info.extend([f"{batch_cost}:ms/batch", f"{row_cost}:μs/row"])
                    pylog.info(", ".join(log_info))
                    log_cnt += 1
                    total_loss = 0.0
                    tic = time.time()
                if step % self.steps_save == 0:
                    save_ckpt(sess, step, self.ckpt_path)
            save_ckpt(sess, step, self.ckpt_path)
            pylog.info(f"train files finish....")
