#!/usr/bin/python
# -*- coding:utf-8 -*-

import importlib
import os
import socket
import subprocess
import sys
import time
import traceback
from datetime import datetime

from .log_util import pylog


def process_error(error, **kwargs):
    pylog.info(f"process_error  error : {error} kwargs : {kwargs}")
    traceback.print_exc(file=sys.stdout)
    raise error


def _get_file_system(dir_or_file_path):
    if dir_or_file_path.startswith("hdfs"):
        return "hdfs"
    else:
        return "local"


def list_files(dir_or_file_path):
    if _get_file_system(dir_or_file_path) == "hdfs":
        files = list_hdfs_path(dir_or_file_path)
        return files
    else:
        files = [os.path.join(dir_or_file_path, x) for x in os.listdir(dir_or_file_path)]
        return files


def get_worker_files(all_files, task_index, n_workers):
    if isinstance(task_index, str):
        tid = int(task_index)
    else:
        tid = task_index
    return [x for i, x in enumerate(all_files) if i % n_workers == tid]


def get_host_ip():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 53))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def shell_cmd(cmd):
    pylog.info(cmd)
    status, out, cnt = -1, str(), 0
    while status != 0 and cnt < 10:
        status, out = subprocess.getstatusoutput(cmd)
        if status == 0:
            pylog.info(f"尝试{cnt}次数 执行状态status={status}")
            break
        pylog.error(f"尝试{cnt}次数 执行状态status={status} 日志out={out}")
        time.sleep(60)
        cnt += 1
    if status != 0:
        pylog.error(f"执行命令失败 cmd={cmd} 执行状态status={status} 日志out={out}")
    return status, out


def get_create_time(path):
    statinfo = os.stat(path)
    create_time = datetime.fromtimestamp(
        statinfo.st_ctime).strftime("%Y%m%d %H:%M:%S")
    return create_time


def load_class(module_name, class_name):
    pylog.info(f"sys.path: {sys.path}")
    pylog.info(f"module_name: {module_name}  class_name={class_name}")
    m = importlib.import_module(module_name)
    c = getattr(m, class_name)
    return c


def load_class_full_class_name(full_class_name):
    try:
        pos = full_class_name.rfind('.')
        module_name = full_class_name[:pos]
        class_name = full_class_name[pos + 1:]
        c = load_class(module_name, class_name)
        return c
    except Exception as e:
        process_error(e)


def list_hdfs_path(hdfs_path):
    cmd = "hadoop fs -ls {}".format(hdfs_path)
    status, out = shell_cmd(cmd)
    if status == 0:
        arr = out.split('\n')
        return [x.split(' ')[-1] for x in arr if len(x.split(' ')) > 4 and "hdfs://" in x]
    return []


if __name__ == "__main__":
    print('hello')
