import tensorflow as tf


def load_pb(model_file_path):
    input_tensor_names, input_op_names, output_tensor_names, output_op_names = [], [], [], []
    session = tf.Session(graph=tf.Graph())
    meta_graph = tf.saved_model.loader.load(session, [tf.saved_model.tag_constants.SERVING], model_file_path)
    model_graph_signature = list(meta_graph.signature_def.items())[0][1]
    for output_item in model_graph_signature.outputs.items():
        output_op_name = output_item[0]
        output_op_names.append(output_op_name)
        output_tensor_name = output_item[1].name
        output_tensor_names.append(output_tensor_name)
    for input_item in model_graph_signature.inputs.items():
        input_op_name = input_item[0]
        input_op_names.append(input_op_name)
        input_tensor_name = input_item[1].name
        input_tensor_names.append(input_tensor_name)
    return session, {"input_tensor_names": input_tensor_names, "input_op_names": input_op_names,
                     "output_tensor_names": output_tensor_names, "output_op_names": output_op_names}


class ReaderPredict:

    def __init__(self, model_file_path):
        session, tensor_op_dict = load_pb(model_file_path)
        self.session = session
        self.tensor_op_dict = tensor_op_dict

    def predict(self, feed_dict):
        del feed_dict["label_ph"]
        feed_dict = {f"import/{k}:0": v for k, v in feed_dict.items()}
        output_values = self.session.run(self.tensor_op_dict["output_tensor_names"], feed_dict=feed_dict)
        output_dict = dict(zip(self.tensor_op_dict["output_op_names"], output_values))
        return output_dict
