#!/usr/bin/python
# -*- coding:utf-8 -*-
# @Time : 2021/7/19 14:26
# @Author : zhl
# @File : base_local_reader.py
import gzip
import os

from dup1.utils.com_util import shell_cmd
from dup1.utils.log_util import pylog


def open_ways(file_path):
    """
    :param file_path:
    :return:
    """
    if file_path.endswith(".gz"):
        process_file = gzip.open(file_path, "r")
    else:
        process_file = open(file_path, "r")
    return process_file


def read_dataset(filename):
    """
    :param filename:
    :return:
    """
    if "hdfs" in filename:
        read_file = f"process_{os.path.basename(filename)}"
        shell_cmd("rm -f process_*gz")
        shell_cmd(f"hadoop fs -get {filename} {read_file}")
        process_file = open_ways(read_file)
    else:
        process_file = open_ways(filename)
    return process_file


class BaseLocalReader:
    def __init__(self, filenames, input_feature, batch_size, **kvargs):
        self.filenames = filenames
        self.input_feature = input_feature
        self.batch_size = batch_size
        self.kvargs = kvargs
        self.cur_file = 0
        self.init_config()

    def __iter__(self):
        return self

    def init_config(self):
        raise NotImplementedError

    def process_line(self, line) -> (int, dict):
        raise NotImplementedError

    def __next__(self):
        if hasattr(self, "source_dataset") is False:
            self.source_dataset = read_dataset(self.filenames[self.cur_file])
        cur, batch_input = 0, {fea_name: [] for fea_name in self.input_feature}
        while cur < self.batch_size:
            line = self.source_dataset.readline()
            if self.filenames[self.cur_file].endswith(".gz"):
                line = line.decode("utf-8")
            if line.strip() != "" and len(line.strip()) > 0:
                row_cnt, row_input = self.process_line(line)
                common_keys = batch_input.keys() & row_input.keys()
                for key in common_keys:
                    for i in range(row_cnt):
                        batch_input.get(key, []).append(row_input[key][i])
                cur += row_cnt
            else:
                pylog.info(f"处理结束 当前第{self.cur_file + 1}/{len(self.filenames)}个数据文件 " +
                           f"文件名字{self.filenames[self.cur_file]}")
                self.cur_file += 1
                if self.cur_file < len(self.filenames):
                    self.source_dataset.close()
                    self.source_dataset = read_dataset(self.filenames[self.cur_file])
                else:
                    raise StopIteration
        return batch_input, {}













