#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：dup1
@File    ：dsin_reader_v1.py
@Author  ：Zhl
@Date    ：2021/9/12 8:00 下午 
"""

import json
import random

from dup1.readers.base_local_reader import BaseLocalReader


class DSINReaderV1(BaseLocalReader):
    def init_config(self):
        kvargs = self.kvargs["kvargs"]
        encode_idx_conf = json.load(open(kvargs["encode_idx_conf"], "r"))
        cid3idx = encode_idx_conf["cid3"]
        cid2idx = encode_idx_conf["cid2"]
        cid1idx = encode_idx_conf["cid1"]
        pwid2idx = encode_idx_conf["pwid"]
        cid3_pwid_tab = encode_idx_conf["cid3_pwid_tab"]
        cid3_pwid_idx = {}
        for cid3, pwid_list in cid3_pwid_tab.items():
            cid3_pwid_idx[cid3idx[cid3]] = [pwid2idx[pwid] for pwid in pwid_list]
        cid3_cids_idx = {}
        cid3_cids_tab = encode_idx_conf["cid3_cids_tab"]
        for cid3, cids in cid3_cids_tab.items():
            for cid in cids:
                cid2, cid1 = cid.split("_")
                cid3_cids_idx[cid3idx[cid3]] = [cid2idx[cid2], cid1idx[cid1]]
        self.cid3idx = cid3idx
        self.cid3_pwid_idx = cid3_pwid_idx
        self.cid3_cids_idx = cid3_cids_idx
        self.act_type = {"order": set([26, 28]), "cart": set([29, 30])}

    def divide_sess(self, sub_seq):
        # time_stamp, act, storeid, event_id, cid1, cid2, cid3, pwid
        last_ts, last_act, _, _, last_cid1, last_cid2, last_cid3, last_pwid = sub_seq[-1]
        last_cid3, last_pwid = int(last_cid3), int(last_pwid)
        cid3_sess_list, cid3_sess = [], [last_cid3]
        pwid_sess_list, pwid_sess = [], [last_pwid]
        for i in range(len(sub_seq) - 2, -1, -1):
            ts, act, _, _, cid1, cid2, cid3, pwid = sub_seq[i]
            cid3, pwid = int(cid3), int(pwid)
            delta = int(last_ts) - int(ts)
            if delta < 30 * 60 * 1000:
                cid3_sess.append(cid3)
                pwid_sess.append(pwid)
            else:
                cid3_sess_list.append(cid3_sess[:10])
                pwid_sess_list.append(pwid_sess[:10])
                cid3_sess = [cid3]
                pwid_sess = [pwid]
            last_ts, last_act, last_cid3, last_pwid = ts, act, cid3, pwid
        cid3_sess_list.append(cid3_sess[:10])
        pwid_sess_list.append(pwid_sess[:10])
        return cid3_sess_list, pwid_sess_list

    def sess_padding(self, sess_list):
        for i in range(5):
            if i < len(sess_list):
                if len(sess_list[i]) < 10:
                    sess_list[i].extend([0 for _ in range(10 - len(sess_list[i]))])
                else:
                    sess_list.append([0 for _ in range(10)])
            else:
                sess_list.append([0 for _ in range(10)])
        return sess_list

    def combine_feed(self, rows_input, cid3, pwid, cid3_sess_list, pwid_sess_list, sess_len, label):
        rows_input["cid3_ph"].append([cid3])
        rows_input["pwid_ph"].append([pwid])
        rows_input["sess_0_cid3_ph"].append(cid3_sess_list[0])
        rows_input["sess_1_cid3_ph"].append(cid3_sess_list[1])
        rows_input["sess_2_cid3_ph"].append(cid3_sess_list[2])
        rows_input["sess_3_cid3_ph"].append(cid3_sess_list[3])
        rows_input["sess_4_cid3_ph"].append(cid3_sess_list[4])
        rows_input["sess_0_pwid_ph"].append(pwid_sess_list[0])
        rows_input["sess_1_pwid_ph"].append(pwid_sess_list[1])
        rows_input["sess_2_pwid_ph"].append(pwid_sess_list[2])
        rows_input["sess_3_pwid_ph"].append(pwid_sess_list[3])
        rows_input["sess_4_pwid_ph"].append(pwid_sess_list[4])
        rows_input["sess_length_ph"].append([sess_len])
        rows_input["label_ph"].append([label])
        return rows_input

    def process_line(self, line) -> dict:
        # 0:time_stamp, 1:act, 2:storeid, 3:event_id, 4:cid1, 5:cid2, 6:cid3, 7:pwid
        rows_input = {fea_name: [] for fea_name in self.input_feature}
        uuid, seq_str = line.replace("\n", "").split("\t")
        seqs = [[c for c in seq.split("_")] for seq in seq_str.split("#")]
        sample = random.choice([i + 2 for i in range(2)])
        for _ in range(sample):
            k = random.choice([idx for idx in range(len(seqs))][len(seqs) // 2:])
            label_ts, label_act, _, label_event, _, _, label_cid3, label_pwid = seqs[k]
            label_cid3, label_pwid = int(label_cid3), int(label_pwid)
            sub_seq = seqs[:k]
            cid3_sess_list, pwid_sess_list = self.divide_sess(sub_seq)
            cid3_pad_list, pwid_pad_list = self.sess_padding(cid3_sess_list), self.sess_padding(pwid_sess_list)
            assert len(cid3_sess_list) == len(pwid_sess_list)
            sess_len = len(cid3_sess_list)
            if int(label_event) in self.act_type["order"]:
                self.combine_feed(rows_input, label_cid3, label_pwid, cid3_pad_list, pwid_pad_list, sess_len, 1)
            else:
                self.combine_feed(rows_input, label_cid3, label_pwid, cid3_pad_list, pwid_pad_list, sess_len, 0)
        return sample, rows_input
