import time

import numpy as np
import tensorflow as tf

from dup1.metirc import metric
from dup1.tasks.task_utils import trans_feat_feed, save_ckpt
from dup1.utils.log_util import pylog


def get_sess_config():
    """
    获取session配置信息
    :return:
    """
    return tf.ConfigProto(
        log_device_placement=True,
        inter_op_parallelism_threads=20,
        intra_op_parallelism_threads=20)


def build_train_op(optim_op, loss_op):
    """
    :param optim_op:
    :param loss_op:
    :return:
    """
    update_ops = tf.get_collection(tf.GraphKeys.UPDATE_OPS)
    step_op = tf.train.get_or_create_global_step()
    with tf.control_dependencies(update_ops):
        train_op = optim_op.minimize(loss_op, global_step=step_op)
    return train_op, step_op, loss_op


class LocalTask:
    def __init__(self, reader, model, ckpt_path, batch_size=256, steps_log=32, steps_metric=64, steps_save=1024):
        """
        :param reader:
        :param model:
        :param ckpt_path:
        :param batch_size:
        :param steps_log:
        :param steps_save:
        """
        self.reader = reader
        self.model = model
        self.ckpt_path = ckpt_path
        self.batch_size = batch_size
        self.steps_log = steps_log
        self.steps_save = steps_save
        self.steps_metric = steps_metric

    def train(self):
        """
        :return:
        """
        with tf.Session(config=get_sess_config()) as sess:
            tf.set_random_seed(9)
            train_op, step_op, loss_op = build_train_op(self.model.optim, self.model.loss)
            pylog.info("Training New Model ...")
            init = [tf.global_variables_initializer()]
            sess.run(init)
            # sess.graph.finalize()
            # some varibles to fomat print
            tic = time.time()
            log_cnt, total_loss = 1, 0.0
            pylog.info(f"train files:{self.reader.filenames}")
            for features, labels in self.reader:
                feed = trans_feat_feed(features, self.model.input_ph_dict)
                _, step, loss, score = sess.run([train_op, step_op, loss_op, self.model.score], feed_dict=feed)
                total_loss += loss
                if step % self.steps_log == 0:
                    toc = time.time()
                    log_info = []
                    batch_cost = round((toc - tic) * 1000 / self.steps_log, 2)
                    row_cost = round((toc - tic) * 1000000 / (self.steps_log * self.batch_size), 2)
                    log_info.extend([f"row:{log_cnt * self.batch_size // 10000}w"])
                    log_info.extend([f"steps:{step // 1000}k", f"log_cnt:{log_cnt}"])
                    log_info.extend([f"loss:{np.round(total_loss / log_cnt, 5)}"])
                    log_info.extend([f"{batch_cost}:ms/batch", f"{row_cost}:μs/row"])
                    pylog.info(", ".join(log_info))
                    log_cnt += 1
                    total_loss = 0.0
                    tic = time.time()
                if step % self.steps_save == 0:
                    save_ckpt(sess, step, self.ckpt_path)
                if step % self.steps_metric == 0:
                    metric(label=features["label_ph"], score=score)
            save_ckpt(sess, step, self.ckpt_path)
            pylog.info(f"train files finish....")
