#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：dup1
@File    ：dsin1_test.py
@Author  ：Zhl
@Date    ：2021/9/10 5:34 下午 
"""

from dup1.feature.feature_column import SparseFeat, VarLenSparseFeat
from dup1.models.dsin import DSIN
from dup1.readers.base_local_reader import BaseLocalReader
from dup1.tasks.local_task import LocalTask


class DSINReader(BaseLocalReader):
    def init_config(self):
        pass

    def process_line(self, line) -> (int, dict):
        row_input = {fea_name: [] for fea_name in self.input_feature}
        cells = line.strip().split(",")
        if len(cells) != 16:
            return 0, row_input
        row_input["age_level_ph"], row_input["cate_id_ph"], = [[int(cells[0])]], [[int(cells[1])]]
        row_input["brand_ph"], row_input["price_ph"] = [[int(cells[2])]], [[float(cells[3])]]
        row_input["sess_0_cate_id_ph"] = [[int(i) for i in cells[4].split("_")]]
        row_input["sess_0_brand_ph"] = [[int(i) for i in cells[5].split("_")]]
        row_input["sess_1_cate_id_ph"] = [[int(i) for i in cells[6].split("_")]]
        row_input["sess_1_brand_ph"] = [[int(i) for i in cells[7].split("_")]]
        row_input["sess_2_cate_id_ph"] = [[int(i) for i in cells[8].split("_")]]
        row_input["sess_2_brand_ph"] = [[int(i) for i in cells[9].split("_")]]
        row_input["sess_3_cate_id_ph"] = [[int(i) for i in cells[10].split("_")]]
        row_input["sess_3_brand_ph"] = [[int(i) for i in cells[11].split("_")]]
        row_input["sess_4_cate_id_ph"] = [[int(i) for i in cells[12].split("_")]]
        row_input["sess_4_brand_ph"] = [[int(i) for i in cells[13].split("_")]]
        row_input["sess_length_ph"] = [[int(cells[14])]]
        row_input["label_ph"] = [[int(cells[15])]]
        return 1, row_input


input_feature = ["age_level_ph", "cate_id_ph", "brand_ph", "price_ph",
                 "sess_0_cate_id_ph", "sess_0_brand_ph", "sess_1_cate_id_ph", "sess_1_brand_ph",
                 "sess_2_cate_id_ph", "sess_2_brand_ph", "sess_3_cate_id_ph", "sess_3_brand_ph",
                 "sess_4_cate_id_ph", "sess_4_brand_ph", "sess_length_ph", "label_ph"]
data_files = ["../../data/train_1.csv"]
batch_size = 256
dsin_reader = DSINReader(filenames=data_files, input_feature=input_feature, batch_size=batch_size)
# for x, y in dsin_reader:
#     print(x, y)

hash_flag = False

feature_columns = [SparseFeat("age_level_ph", 6 + 10, use_hash=hash_flag),
                   SparseFeat("cate_id_ph", 8520 + 100, use_hash=hash_flag),
                   SparseFeat("brand_ph", 146619 + 1000, use_hash=hash_flag)]
# ,DenseFeat("price_ph", 1)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_0_cate_id_ph", 8520 + 100, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cate_id_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_0_brand_ph', 146619 + 1000, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='brand_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_1_cate_id_ph", 8520 + 100, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cate_id_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_1_brand_ph', 146619 + 1000, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='brand_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_2_cate_id_ph", 8520 + 100, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cate_id_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_2_brand_ph', 146619 + 1000, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='brand_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_3_cate_id_ph", 8520 + 100, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cate_id_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_3_brand_ph', 146619 + 1000, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='brand_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_4_cate_id_ph", 8520 + 100, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cate_id_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_4_brand_ph', 146619 + 1000, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='brand_ph'), maxlen=10)]

behavior_feature_list = ["cate_id_ph", "brand_ph"]
sess_count = 5
sess_len_max = 10
dsin_model = DSIN(feature_columns, behavior_feature_list, sess_max_count=sess_count, dnn_hidden_units=(200, 80),
                  att_head_num=8,
                  att_embedding_size=1, bias_encoding=False)

ckpt_path = "../../output/model2ckpt"
local_task = LocalTask(reader=dsin_reader, model=dsin_model, ckpt_path=ckpt_path, steps_log=32)
local_task.train()
