#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：dup1
@File    ：dsin1_test.py
@Author  ：Zhl
@Date    ：2021/9/10 5:34 下午 
"""

from dup1.feature.feature_column import SparseFeat, VarLenSparseFeat
from dup1.models.dsin import DSIN
from dup1.tasks.local_task import LocalTask
from test.readers.dsin_reader_v1_test import DSINReader

input_feature = ["cid3_ph", "pwid_ph",
                 "sess_0_cid3_ph", "sess_0_pwid_ph", "sess_1_cid3_ph", "sess_1_pwid_ph",
                 "sess_2_cid3_ph", "sess_2_pwid_ph", "sess_3_cid3_ph", "sess_3_pwid_ph",
                 "sess_4_cid3_ph", "sess_4_pwid_ph", "sess_length_ph", "label_ph"]
batch_size = 256
kvargs = {"encode_idx_conf": "../../data/encode_idx_conf2.json"}
data_files = ["../../data/000279_0.txt"]
dsin_reader = DSINReader(filenames=data_files, input_feature=input_feature, batch_size=batch_size, kvargs=kvargs)

# for x, y in dsin_reader:
#     for key, val in x.items():
#         if "sess" in key and "length" not in key:
#             for i, v in enumerate(val):
#                 if len(v) != 10:
#                     print(i, key, val)

hash_flag = False

feature_columns = [SparseFeat("cid3_ph", 4891, use_hash=hash_flag),
                   SparseFeat("pwid_ph", 41453, use_hash=hash_flag)]
# ,DenseFeat("price_ph", 1)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_0_cid3_ph", 4891, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cid3_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_0_pwid_ph', 41453, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='pwid_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_1_cid3_ph", 4891, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cid3_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_1_pwid_ph', 41453, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='pwid_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_2_cid3_ph", 4891, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cid3_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_2_pwid_ph', 41453, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='pwid_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_3_cid3_ph", 4891, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cid3_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat("sess_3_pwid_ph", 41453, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='pwid_ph'), maxlen=10)]
feature_columns += [
    VarLenSparseFeat(SparseFeat("sess_4_cid3_ph", 4891, embedding_dim=4,
                                use_hash=hash_flag, embedding_name="cid3_ph"), maxlen=10),
    VarLenSparseFeat(SparseFeat('sess_4_pwid_ph', 41453, embedding_dim=4,
                                use_hash=hash_flag, embedding_name='pwid_ph'), maxlen=10)]

behavior_feature_list = ["cid3_ph", "pwid_ph"]
sess_count = 5
sess_len_max = 10
dsin_model = DSIN(feature_columns, behavior_feature_list, sess_max_count=sess_count, dnn_hidden_units=(200, 80),
                  att_head_num=8,
                  att_embedding_size=1, bias_encoding=False)

ckpt_path = "../../output/model2ckpt"
local_task = LocalTask(reader=dsin_reader, model=dsin_model, ckpt_path=ckpt_path, steps_log=32)
local_task.train()
