#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：dup1
@File    ：dsin_reader_v1_test.py
@Author  ：Zhl
@Date    ：2021/9/11 8:48 上午 
"""
import json
import random

from dup1.readers.base_local_reader import BaseLocalReader


class DSINReader(BaseLocalReader):
    def init_config(self):
        kvargs = self.kvargs["kvargs"]
        encode_idx_conf = json.load(open(kvargs["encode_idx_conf"], "r"))
        cid3idx = list(encode_idx_conf["cid3"].values())
        cid32idx = encode_idx_conf["cid3"]
        pwid2idx = encode_idx_conf["pwid"]
        cid3_pwid_tab = encode_idx_conf["cid3_pwid_tab"]
        cid3_pwid_idx = {}
        for cid3, pwid_list in cid3_pwid_tab.items():
            key = cid32idx[cid3]
            vals = [pwid2idx[pwid] for pwid in pwid_list]
            cid3_pwid_idx[key] = vals
        self.cid3idx = cid3idx
        self.cid3_pwid_idx = cid3_pwid_idx

    def divide_sess(self, sub_seq):
        # time_stamp, act, storeid, event_id, cid3, pwid
        last_ts, last_act, _, _, last_cid3, last_pwid = sub_seq[-1]
        last_cid3, last_pwid = int(last_cid3), int(last_pwid)
        cid3_sess_list, cid3_sess = [], [last_cid3]
        pwid_sess_list, pwid_sess = [], [last_pwid]
        for i in range(len(sub_seq) - 2, -1, -1):
            ts, act, _, _, cid3, pwid = sub_seq[i]
            cid3, pwid = int(cid3), int(pwid)
            delta = int(last_ts) - int(ts)
            if delta < 30 * 60 * 1000:
                cid3_sess.append(cid3)
                pwid_sess.append(pwid)
            else:
                pwid_sess_list.append(pwid_sess[:10])
                cid3_sess_list.append(cid3_sess[:10])
                cid3_sess = [cid3]
                pwid_sess = [pwid]
            last_ts, last_act, last_cid3, last_pwid = ts, act, cid3, pwid
        cid3_sess_list.append(cid3_sess[:10])
        pwid_sess_list.append(pwid_sess[:10])
        return cid3_sess_list, pwid_sess_list

    def sess_padding(self, sess_list):
        for i in range(5):
            if i < len(sess_list):
                if len(sess_list[i]) < 10:
                    sess_list[i].extend([0 for _ in range(10 - len(sess_list[i]))])
                else:
                    sess_list.append([0 for _ in range(10)])
            else:
                sess_list.append([0 for _ in range(10)])
        return sess_list

    def neg_sampling(self):
        cid3idx = self.cid3idx
        cid3_pwid_idx = self.cid3_pwid_idx
        neg_sess_len = [random.randint(0, 9) for i in range(5)]
        cid3_neg_list, pwid_neg_list = [], []
        neg_length = 0
        for i in range(5):
            cid3_list, pwid_list = [], []
            if neg_sess_len[i] > 0:
                neg_length += 1
            for j in range(neg_sess_len[i]):
                cid3 = random.choice(cid3idx)
                cid3_list.append(cid3)
                pwid_list.append(random.choice(cid3_pwid_idx[cid3]))
            for k in range(10 - neg_sess_len[i]):
                cid3_list.append(0)
                pwid_list.append(0)
            cid3_neg_list.append(cid3_list)
            pwid_neg_list.append(pwid_list)
        cid3_neg = random.choice(cid3idx)
        pwid_neg = random.choice(cid3_pwid_idx[cid3_neg])
        return cid3_neg, pwid_neg, cid3_neg_list, pwid_neg_list, neg_length

    def process_line(self, line) -> dict:
        rows_input = {fea_name: [] for fea_name in self.input_feature}
        rows_uuid, rows_cid3, rows_pwid, rows_sess_length, rows_label = [], [], [], [], []
        rows_sess0_cid3, rows_sess1_cid3, rows_sess2_cid3, rows_sess3_cid3, rows_sess4_cid3 = [], [], [], [], []
        rows_sess0_pwid, rows_sess1_pwid, rows_sess2_pwid, rows_sess3_pwid, rows_sess4_pwid = [], [], [], [], []
        uuid, seq_str = line.replace("\n", "").split("\t")
        seqs = [[c for c in seq.split("_")] for seq in seq_str.split("#")]
        # 正样本采样
        pos = random.choice([i + 2 for i in range(2)])
        for i in range(pos):
            k = random.choice([idx for idx in range(len(seqs))][len(seqs) // 2:])
            if len(seqs) == 1:
                selected = seqs[k]
                sub_seq = seqs
            else:
                selected = seqs[k]
                sub_seq = seqs[:k]
            ts, act, _, _, cid3_pos, pwid_pos = selected
            cid3_sess_list, pwid_sess_list = self.divide_sess(sub_seq)
            assert len(cid3_sess_list) == len(pwid_sess_list)
            pos_sess_len = len(cid3_sess_list)
            cid3_pos_list, pwid_pos_list = self.sess_padding(cid3_sess_list), self.sess_padding(pwid_sess_list)
            rows_uuid.append([uuid])
            rows_cid3.append([int(cid3_pos)])
            rows_pwid.append([int(pwid_pos)])
            rows_sess0_cid3.append(cid3_pos_list[0])
            rows_sess1_cid3.append(cid3_pos_list[1])
            rows_sess2_cid3.append(cid3_pos_list[2])
            rows_sess3_cid3.append(cid3_pos_list[3])
            rows_sess4_cid3.append(cid3_pos_list[4])
            rows_sess0_pwid.append(pwid_pos_list[0])
            rows_sess1_pwid.append(pwid_pos_list[1])
            rows_sess2_pwid.append(pwid_pos_list[2])
            rows_sess3_pwid.append(pwid_pos_list[3])
            rows_sess4_pwid.append(pwid_pos_list[4])
            rows_sess_length.append([pos_sess_len])
            rows_label.append([1])

        # 负样本采样
        neg = random.choice([i + 4 for i in range(4)])
        for i in range(neg):
            cid3_neg, pwid_neg, cid3_neg_list, pwid_neg_list, neg_sess_len = self.neg_sampling()
            rows_uuid.append([uuid])
            rows_cid3.append([int(cid3_neg)])
            rows_pwid.append([int(pwid_neg)])
            rows_sess0_cid3.append(cid3_neg_list[0])
            rows_sess1_cid3.append(cid3_neg_list[1])
            rows_sess2_cid3.append(cid3_neg_list[2])
            rows_sess3_cid3.append(cid3_neg_list[3])
            rows_sess4_cid3.append(cid3_neg_list[4])
            rows_sess0_pwid.append(pwid_neg_list[0])
            rows_sess1_pwid.append(pwid_neg_list[1])
            rows_sess2_pwid.append(pwid_neg_list[2])
            rows_sess3_pwid.append(pwid_neg_list[3])
            rows_sess4_pwid.append(pwid_neg_list[4])
            rows_sess_length.append([neg_sess_len])
            rows_label.append([0])

        rows_input["cid3_ph"] = rows_cid3
        rows_input["pwid_ph"] = rows_pwid
        rows_input["sess_0_cid3_ph"] = rows_sess0_cid3
        rows_input["sess_1_cid3_ph"] = rows_sess1_cid3
        rows_input["sess_2_cid3_ph"] = rows_sess2_cid3
        rows_input["sess_3_cid3_ph"] = rows_sess3_cid3
        rows_input["sess_4_cid3_ph"] = rows_sess4_cid3
        rows_input["sess_0_pwid_ph"] = rows_sess0_pwid
        rows_input["sess_1_pwid_ph"] = rows_sess1_pwid
        rows_input["sess_2_pwid_ph"] = rows_sess2_pwid
        rows_input["sess_3_pwid_ph"] = rows_sess3_pwid
        rows_input["sess_4_pwid_ph"] = rows_sess4_pwid
        rows_input["label_ph"] = rows_label
        rows_input["sess_length_ph"] = rows_sess_length
        return neg + pos, rows_input


# # time_stamp, act, storeid, event_id, cid3, pwid
# input_feature = ["cid3_ph", "pwid_ph",
#                  "sess_0_cid3_ph", "sess_0_pwid_ph", "sess_1_cid3_ph", "sess_1_pwid_ph",
#                  "sess_2_cid3_ph", "sess_2_pwid_ph", "sess_3_cid3_ph", "sess_3_pwid_ph",
#                  "sess_4_cid3_ph", "sess_4_pwid_ph", "sess_length_ph", "label_ph"]
# batch_size = 256
# kvargs = {"encode_idx_conf": "../../data/encode_idx_conf2.json"}
# data_files = ["../../data/000279_0.txt"]
# dsin_reader = DSINReader(filenames=data_files, input_feature=input_feature, batch_size=batch_size, kvargs)
# for x, y in dsin_reader:
#     print(x, y)
